import { type Route } from '@/schemas';
import { useUserStore } from '@/stores';
import { watch, type Component } from 'vue';
import { createRouter, createWebHistory, type RouteRecordRaw } from 'vue-router';
const routes: RouteRecordRaw[] = [
  {
    path: '/',
    redirect: '/home'
  },
  {
    path: '/404',
    component: () => import('@/views/NotFoundPage.vue')
  }
];
const router = createRouter({
  history: createWebHistory(),
  routes: routes
});
const componentMap: Record<string, () => Promise<Component>> = {
  'MainPage.vue': () => import('@/views/MainPage.vue')
};
function _getAvailableRoutes(routeResponses: Route[], root: boolean, pathList: string[]) {
  const res: RouteRecordRaw[] = [];
  const { routeMap } = useUserStore();
  for (const routeResponse of routeResponses) {
    const path = routeResponse.name.toLowerCase();
    pathList.push(path);
    const route = {
      path: root ? `/${path}` : path,
      component: componentMap[routeResponse.component],
      children: _getAvailableRoutes(routeResponse.children, false, pathList.concat(path))
    };
    routeMap.set(`/${pathList.join('/')}`, root ? route : routeMap.get(`/${pathList[0]}`)!);
    res.push(route);
  }
  return res;
}
export function getAvailableRoutes(routeResponse: Route[]) {
  _getAvailableRoutes(routeResponse, true, []);
}
router.beforeEach(async (to) => {
  const userStore = useUserStore();
  console.log(2, userStore.userInfo);
  // await timeout(1000);
  console.log(userStore.initialized);
  if (!userStore.initialized) {
    await new Promise((resolve) => watch(() => userStore.initialized, resolve));
  }
  if (userStore.addedRouteSet.has(to.fullPath) || to.fullPath === '/404') {
    return true;
  }
  const route = userStore.routeMap.get(to.fullPath);
  if (route !== undefined) {
    router.addRoute(route);
    userStore.addedRouteSet.add(to.fullPath);
    return to;
  } else {
    return '/404';
  }
});
export default router;
